<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

use Rebilly\Entities\PaymentCard;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Exception\AjaxResponseException;
use Rebilly\Wp\Member\Plugin;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class CardDeactivate
 * @package Rebilly\Wp\Member\Action
 */
class CardDeactivate extends AjaxAction
{
    public function __construct()
    {
        parent::__construct('rebilly_card_deactivate', false);
    }

    /**
     * @param array $data
     *
     * @throws AjaxResponseException
     * @return array
     */
    public function process(array $data): array
    {
        $user = User::find();
        if (!$user) {
            $this->fail('You must be authorized to deactivate any payment card');
        }
        if (!$user->getCustomerId()) {
            // Actually no payment cards at all, but no need to inform about it
            $this->fail('Payment Card not found');
        }
        $paymentCard = $this->getCard($data);
        if ($paymentCard->getCustomerId() !== $user->getCustomerId()) {
            $this->fail('Payment Card not found');
        }
        if ($paymentCard->getStatus() === 'deactivated') {
            $this->fail('Payment Card not found');
        }

        if ($this->deactivateCard($paymentCard)) {
            $this->success('Your payment card is deactivated');
        }

        return [
            'success' => false,
            'message' => __('Unexpected error, try again', REBILLY_WP_LANG_DOMAIN),
        ];
    }

    protected function verifyNonce(array $data): bool
    {
        return !$this->useNonce || wp_verify_nonce($data['_nonce'], $this->name . $data['paymentCardId']);
    }

    /**
     * @param array $data
     *
     * @throws \Rebilly\Wp\Member\Exception\AjaxFailException
     * @return PaymentCard
     */
    private function getCard(array $data): PaymentCard
    {
        if (!($card = Plugin::api()->paymentCard()->get($data['paymentCardId']))) {
            $this->fail('Payment Card not found');
        }

        return $card;
    }

    /**
     * @param PaymentCard $card
     *
     * @return bool
     */
    private function deactivateCard(PaymentCard $card): bool
    {
        $result = Plugin::api()
            ->paymentCard()
            ->deactivate($card->getId());

        return $result && $result->getStatus() === 'deactivated';
    }
}
