<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

use Rebilly\Entities\Customer;
use Rebilly\Entities\PaymentCard;
use Rebilly\Entities\PaymentInstruments\PaymentCardInstrument;
use Rebilly\Http\Exception\UnprocessableEntityException;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Exception\AjaxResponseException;
use Rebilly\Wp\Member\Plugin;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class CardUpdate
 * @package Rebilly\Wp\Member\Action
 */
class CardUpdate extends AjaxAction
{
    public function __construct()
    {
        parent::__construct('rebilly_card_update', false);
    }

    /**
     * @param array $data
     *
     * @throws AjaxResponseException
     * @return array
     */
    public function process(array $data): array
    {
        $user = User::find();
        if (!$user) {
            $this->fail('You must be authorized to update any payment card');
        }
        if (!$user->getCustomerId()) {
            // Actually no payment cards at all, but no need to inform about it
            $this->fail('Payment Card not found');
        }
        $paymentCard = $this->getCard($data);
        if ($paymentCard->getCustomerId() !== $user->getCustomerId()) {
            $this->fail('Payment Card not found');
        }
        if ($paymentCard->getStatus() === 'deactivated') {
            $this->fail('Payment Card not found');
        }

        if ($this->updateCard($data, $user, $paymentCard)) {
            $this->success('Your payment card is updated');
        }

        return [
            'success' => false,
            'message' => __('Unexpected error, try again', REBILLY_WP_LANG_DOMAIN),
        ];
    }

    protected function verifyNonce(array $data): bool
    {
        return !$this->useNonce || wp_verify_nonce($data['_nonce'], $this->name . $data['paymentCardId']);
    }

    /**
     * @param array $data
     *
     * @throws \Rebilly\Wp\Member\Exception\AjaxFailException
     * @return PaymentCard
     */
    private function getCard(array $data): PaymentCard
    {
        if (!($card = Plugin::api()->paymentCard()->get($data['paymentCardId']))) {
            $this->fail('Payment Card not found');
        }

        return $card;
    }

    /**
     * @param array $data
     * @param User $user
     * @param PaymentCard $card
     *
     * @throws \Rebilly\Wp\Member\Exception\AjaxFailException
     * @return bool
     */
    private function updateCard(array $data, User $user, PaymentCard $card): bool
    {
        try {
            if (isset($data['expMonth'], $data['expYear'])) {
                $card->setExpMonth($data['expMonth'])
                    ->setExpYear($data['expYear']);
                Plugin::api()
                    ->paymentCard()
                    ->update($card->getId(), $card);
            }
            if (isset($data['status']) && $data['status'] === 'active') {
                Plugin::api()
                    ->customer()
                    ->update(
                        $user->getCustomerId(),
                        (new Customer())
                            ->setDefaultPaymentInstrument(
                                (new PaymentCardInstrument())
                                    ->setPaymentCardId($card->getId())
                            )
                    );
            }
        } catch (UnprocessableEntityException $e) {
            $this->fail($e->getMessage(), $e->getErrors());
        }

        return true;
    }
}
