<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class Login
 * @package Rebilly\Wp\Member\Action
 */
class Login extends AjaxAction
{
    public function __construct()
    {
        parent::__construct('rebilly_form_login', true);
    }

    /**
     * Before wp_signon we imitate logic of wp-login.php
     *
     * @param array $data
     *
     * @return array
     */
    public function process(array $data): array
    {
        $secure_cookie = '';

        // If the user wants SSL but the session is not SSL, force a secure cookie.
        if (!empty($data['username']) && !force_ssl_admin()) {
            $user_name = sanitize_user($data['username']);
            $user = get_user_by('login', $user_name);

            if (!$user && strpos($user_name, '@')) {
                $user = get_user_by('email', $user_name);
            }

            if ($user && get_user_option('use_ssl', $user->ID)) {
                $secure_cookie = true;
                force_ssl_admin(true);
            }
        }

        if (isset($data['redirect_to'])) {
            $redirect_to = $data['redirect_to'];
            // Redirect to HTTPS if user wants SSL.
            if ($secure_cookie && false !== strpos($redirect_to, 'wp-admin')) {
                $redirect_to = preg_replace('|^http://|', 'https://', $redirect_to);
            }
        } else {
            $redirect_to = admin_url();
        }

        $user = wp_signon([
            'user_login' => $data['username'],
            'user_password' => $data['password'],
            'remember' => $data['remember'],
        ], $secure_cookie);

        $requested_redirect_to = $data['redirect_to'] ?? '';

        $redirect_to = apply_filters('login_redirect', $redirect_to, $requested_redirect_to, $user);

        if (!is_wp_error($user)) {
            if (empty($redirect_to) || $redirect_to === 'wp-admin/' || $redirect_to === admin_url()) {
                // If the user doesn't belong to a blog, send them to user admin. If the user can't edit posts, send them to their profile.
                if (is_multisite()) {
                    if (!get_active_blog_for_user($user->ID) && !is_super_admin($user->ID)) {
                        $redirect_to = user_admin_url();
                    } elseif (!$user->has_cap('read')) {
                        $redirect_to = get_dashboard_url($user->ID);
                    }
                } elseif (!$user->has_cap('edit_posts')) {
                    $redirect_to = $user->has_cap('read') ? admin_url('profile.php') : home_url();
                }
            }

            return [
                'success' => true,
                'redirect' => $redirect_to,
            ];
        }

        return [
            'success' => false,
            'errors' => [
                'password' => __('The password you entered is incorrect.', REBILLY_WP_LANG_DOMAIN),
            ],
        ];
    }
}
