<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

use Rebilly\Entities\Subscription;
use Rebilly\Entities\SubscriptionCancellation;
use Rebilly\Http\Exception\UnprocessableEntityException;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Exception\AjaxResponseException;
use Rebilly\Wp\Member\Plugin;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class SubscriptionCancel
 * @package Rebilly\Wp\Member\Action
 */
class SubscriptionCancel extends AjaxAction
{
    public function __construct()
    {
        parent::__construct('rebilly_subscription_cancel', true);
    }

    /**
     * @param array $data
     *
     * @throws AjaxResponseException
     * @return array
     */
    public function process(array $data): array
    {
        $user = User::find();
        if (!$user) {
            $this->fail('You must be authorized to cancel any subscription');
        }
        if (!$user->getCustomerId()) {
            // Actually no subscriptions at all, but no need to inform about it
            $this->fail('Subscription not found');
        }

        $subscription = $this->getSubscription($data);
        if ($subscription->getCustomerId() !== $user->getCustomerId()) {
            $this->fail('Subscription not found');
        }
        if (!in_array($subscription->getStatus(), ['active', 'pending'], true)) {
            $this->fail('Subscription is not active');
        }

        if ($this->cancelSubscription($data, $subscription)) {
            $this->success('Your subscription is cancelled');
        }

        return [
            'success' => false,
            'message' => __('Unexpected error, try again', REBILLY_WP_LANG_DOMAIN),
        ];
    }

    protected function verifyNonce(array $data): bool
    {
        return !$this->useNonce || wp_verify_nonce($data['_nonce'], $this->name . $data['subscriptionId']);
    }

    /**
     * @param array $data
     *
     * @throws \Rebilly\Wp\Member\Exception\AjaxFailException
     * @return Subscription
     */
    private function getSubscription(array $data): ?Subscription
    {
        if (!($subscription = Plugin::api()->subscription()->get($data['subscriptionId']))) {
            $this->fail('Subscription not found');
        }

        return $subscription;
    }

    /**
     * @param array $data
     * @param Subscription $subscription
     *
     * @throws \Rebilly\Wp\Member\Exception\AjaxFailException
     * @return bool
     */
    private function cancelSubscription(array $data, Subscription $subscription): bool
    {
        $cancellation = (new SubscriptionCancellation())
            ->setSubscriptionId($subscription->getId())
            ->setStatus(SubscriptionCancellation::STATUS_CONFIRMED)
            ->setCanceledBy(SubscriptionCancellation::SOURCE_CUSTOMER)
            ->setReason($data['reason'] ?? SubscriptionCancellation::CATEGORY_OTHER)
            ->setDescription($data['description'] ?? null);

        // TODO two strategies based on settings
        // 1. churnTime = renewalTime & prorated = false
        // 2. churnTime = now() & prorated = true
        $cancellation->setChurnTime($subscription->getRenewalTime())
            ->setProrated(false);

        try {
            return Plugin::api()
                    ->subscription()
                    ->cancel($cancellation) !== null;
        } catch (UnprocessableEntityException $e) {
            $this->fail('Unexpected error');
        }

        return false;
    }
}
