<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

namespace Rebilly\Wp\Member\Admin;

use Rebilly\Wp\Member\Entity\Capability;
use Rebilly\Wp\Member\Entity\Form;
use Rebilly\Wp\Member\Entity\RestrictableType as Restrictable;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Settings;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class RestrictableType implements Metaboxable
{
    use HasMetabox;

    private $metabox_id = 'wp_rebilly_m_capabilities';

    private $columnId = 'wp_rebilly_m__status';

    public function __construct()
    {
        $this->metabox_init([
            'id' => 'wp_rebilly_m_capabilities',
            'label' => __('Rebilly Content Protection', REBILLY_WP_LANG_DOMAIN),
            'screen' => Settings::get('post_types', []),
            'context' => 'side',
            'priority' => 'high',
            'classes' => ['wp_rebilly_m__metabox'],
            '__block_editor_compatible_meta_box' => false,
        ]);
        add_action('admin_head', [$this, 'add_mce_button']);
        add_action('after_wp_tiny_mce', [$this, 'populate_mce_button']);

        /**
         * Thrive Architect removes all MCE hooks and we need to init them back
         */
        add_action('tcb_hook_template_redirect', [$this, 'add_mce_button']);

        // Posts Table columns
        add_filter('manage_posts_columns', [$this, 'tableColumn']);
        add_action('manage_posts_custom_column', [$this, 'tableColumnContent'], 10, 2);

        // Pages Table columns
        add_filter('manage_pages_columns', [$this, 'tableColumn']);
        add_action('manage_pages_custom_column', [$this, 'tableColumnContent'], 10, 2);
    }

    public function populate_mce_button()
    {
        $formsRaw = Form::all() ?? [];
        $forms = [];
        foreach ($formsRaw as $form) {
            $forms[] = [
                'value' => $form->ID,
                'label' => get_the_title($form),
            ];
        }
        /** @noinspection JSUnresolvedVariable */
        /** @noinspection BadExpressionStatementJS */
        printf('<script type="text/javascript">window.wp_rebilly_m = window.wp_rebilly_m || {};window.wp_rebilly_m.forms = %s;</script>', json_encode($forms));
    }

    public function add_mce_button()
    {
        if (!current_user_can('edit_posts') && !current_user_can('edit_pages')) {
            return;
        }
        if (get_user_option('rich_editing') == 'true') {
            add_filter('mce_external_plugins', [$this, 'add_mce_plugin']);
            add_filter('mce_buttons', [$this, 'register_mce_button']);
        }
    }

    public function register_mce_button($buttons)
    {
        array_push($buttons, 'wp_rebilly_m-combined');

        return $buttons;
    }

    public function add_mce_plugin($plugin_array)
    {
        $plugin_array['wp_rebilly_m'] = REBILLY_WP_URL . 'dist/js/mce-button.js';

        return $plugin_array;
    }

    public function init_metabox()
    {
        add_action('add_meta_boxes', [$this, 'add_metabox']);
        add_action('save_post', [$this, 'save_metabox'], 10, 2);
    }

    public function metabox_fields($metabox_id, $post)
    {
        $products = Capability::getNames();
        $members = User::getNames();

        return [
            [
                'label_checked' => '<i class="dashicons dashicons-lock color-green"></i> ' . __('Protection enabled', REBILLY_WP_LANG_DOMAIN),
                'label_unchecked' => '<i class="dashicons dashicons-unlock color-red"></i> ' . __('Protection disabled', REBILLY_WP_LANG_DOMAIN),
                'name' => 'status',
                'type' => 'checkbox-inline',
                'esc_labels' => false,
            ],
            [
                'label' => __('Restrict access to Subscribers to any of these Products', REBILLY_WP_LANG_DOMAIN),
                'name' => 'capabilities',
                'type' => 'select',
                'multiple' => true,
                'options' => $products,
                'condition' => [
                    'status' => true,
                ],
            ],
            [
                'label' => __('Allow access to these members', REBILLY_WP_LANG_DOMAIN),
                'name' => 'members',
                'type' => 'select',
                'multiple' => true,
                'options' => $members,
                'condition' => [
                    'status' => true,
                ],
            ],
        ];
    }

    public function metabox_data($metabox_id, $post)
    {
        return Restrictable::find($post)
            ->metaAll(true);
    }

    public function metabox_save_fields($metabox_id, $post, $fields)
    {
        Restrictable::find($post)
            ->metaUpdate($fields, true);
    }

    /**
     * Adds protection status to Restrictable Type Table columns.
     *
     * @since 2.8.3
     *
     * @global object $wpmem The WP-Members Object
     * @param  array $columns The array of table columns.
     * @return array  $columns
     */
    public function tableColumn($columns)
    {
        $post_type = (isset($_REQUEST['post_type'])) ? sanitize_text_field($_REQUEST['post_type']) : 'post';

        if (in_array($post_type, Settings::get('post_types'), true)) {
            $columns[$this->columnId] = __('Rebilly Content Protection', REBILLY_WP_LANG_DOMAIN);
        }

        return $columns;
    }

    /**
     * Adds protection status to the Restrictable Type Table column.
     *
     * @param string $columnId
     * @param int $postId
     */
    public function tableColumnContent($columnId, $postId)
    {
        $post_type = (isset($_REQUEST['post_type'])) ? sanitize_text_field($_REQUEST['post_type']) : 'post';

        if ($columnId == $this->columnId && in_array($post_type, Settings::get('post_types'), true)) {
            $restrictable = Restrictable::find($postId);
            $isProtected = $restrictable && $restrictable->isProtected();
            printf(
                '<span class="dashicons dashicons-%s color-%s" title="%s"></span>',
                $isProtected ? 'lock' : 'unlock',
                $isProtected ? 'green' : 'red',
                __($isProtected ? 'Protection enabled' : 'Protection disabled', REBILLY_WP_LANG_DOMAIN)
            );
            if ($isProtected) {
                $names = Capability::getNames();
                echo implode(
                    ', ',
                    array_map(function ($capability) use ($names) {
                        return $names[$capability] ?? $capability;
                    }, $restrictable->getRestrictions())
                );
            }
        }
    }
}
