<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Api;

use Exception;
use Rebilly\Entities\RulesEngine\Actions\SendEmail;
use Rebilly\Entities\RulesEngine\Actions\SendEmail\EmailNotification;
use Rebilly\Entities\RulesEngine\Actions\SendEmail\EmailTemplate;
use Rebilly\Entities\RulesEngine\Bind;
use Rebilly\Entities\RulesEngine\Condition;
use Rebilly\Entities\RulesEngine\EventRules;
use Rebilly\Entities\RulesEngine\RuleAction;
use Rebilly\Http\Exception\NotFoundException;
use Rebilly\Http\Exception\UnprocessableEntityException;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class RulesEngineService
 * @package Rebilly\Wp\Member\Api
 */
class RulesEngineService extends Service
{
    public const EVENT_SUBSCRIPTION_ACTIVATED = 'subscription-activated';

    /**
     * @param string $eventType
     *
     * @return EventRules|null
     */
    public function get(string $eventType): ?EventRules
    {
        if (!blank($eventType)) {
            try {
                return $this->api()
                    ->eventRules()
                    ->load($eventType);
            } catch (Exception $ex) {
            }
        }

        return null;
    }

    /**
     * @param string $eventType
     * @param EventRules|array $data
     *
     * @throws UnprocessableEntityException
     * @return EventRules|null
     */
    public function update(string $eventType, $data): ?EventRules
    {
        try {
            return $this->api()
                ->eventRules()
                ->update($eventType, $data);
        } catch (Exception $ex) {
            if ($ex instanceof UnprocessableEntityException) {
                throw $ex;
            }
        }

        return null;
    }

    /**
     * @param string $websiteId
     * @param array $fields
     *
     * @throws NotFoundException
     * @return string|null
     */
    public function registerConfirmationEmail(string $websiteId, array $fields): ?string
    {
        $rules = $this->api()
            ->eventRules()
            ->load(self::EVENT_SUBSCRIPTION_ACTIVATED);

        $bind = self::confirmationEmailBind($websiteId, $fields);
        $binds = $rules->getBinds();

        foreach ($binds as $existingBind) {
            if ($existingBind->getName() === $bind->getName()) {
                return $bind->getName();
            }
        }

        $binds[] = $bind;
        $rules->setBinds($binds);

        $this->api()
            ->eventRules()
            ->update(self::EVENT_SUBSCRIPTION_ACTIVATED, $rules);

        return $bind->getName();
    }

    /**
     * @param string $websiteId
     * @param array $fields
     *
     * @return Bind
     */
    public static function confirmationEmailBind(string $websiteId, array $fields): Bind
    {
        return (new Bind())
            ->setName("wp-{$websiteId}-confirmation-email")
            ->setStatus(Bind::STATUS_ACTIVE)
            ->setCriteria(
                (new Condition())
                    ->setOp(Condition::OP_AND)
                    ->setConditions([
                        (new Condition())
                            ->setOp(Condition::OP_EQUALS)
                            ->setPath('/subscription/websiteId')
                            ->setValue($websiteId),
                        (new Condition())
                            ->setOp(Condition::OP_EQUALS)
                            ->setPath("/subscription/customFields/{$fields['email-confirmation']}")
                            ->setValue(true),
                    ])
            )
            ->setActions([
                (new SendEmail())
                    ->setStatus(RuleAction::STATUS_ACTIVE)
                    ->setTitle('WordPress Confirmation Email')
                    ->setEmails([
                        (new EmailNotification())
                            ->setTemplates([
                                (new EmailTemplate())
                                    ->setLocale('en-US')
                                    ->setFrom('{{ subscription.website.serviceEmail }}')
                                    ->setTo(['{{ subscription.customer.primaryAddress.emails.0.value }}'])
                                    ->setSubject('Welcome to {{ subscription.website.name }}')
                                    ->setText(
                                        <<<TEXT
Thank you for subscribing to {{ subscription.website.url }}!
We created an account for you {{subscription.customer.customFields.{$fields['username']}}}.

To activate your account follow this link {{subscription.customFields.{$fields['email-confirmation-url']}}}
TEXT
                                    )
                                    ->setHtml(
                                        <<<HTML
<!DOCTYPE html>
<html lang="en" style="display: block; height: 100%;">
<head>
  <title>Your subscription was activated</title>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <meta name="viewport" content="width=device-width">
</head>
<body style="color: #0D2B3E; display: block; font-family: Roboto, sans-serif; font-size: 14px; height: 100%; margin: 0; padding: 20px;">
  <h3>Thank you for subscribing to {{ subscription.website.url }}!</h3>
  <p>We created an account for you <strong>{{subscription.customer.customFields.{$fields['username']}}}</strong>.</p>
  <p>To activate your account click <a href="{{subscription.customFields.{$fields['email-confirmation-url']}}}">here</a></p>
  <p>Or follow this link {{subscription.customFields.{$fields['email-confirmation-url']}}}</p>
</body>
</html>
HTML
                                    ),
                            ])
                            ->setWeight(100),
                    ]),
            ]);
    }
}
