<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Entity;

use InvalidArgumentException;
use WP_Post;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

trait PostProxy
{
    /**
     * @var WP_Post
     */
    protected $post;

    /**
     * PostProxy constructor.
     * @param WP_Post $post
     */
    public function __construct($post)
    {
        if (!($post instanceof WP_Post) || !static::validatePost($post)) {
            $class = get_class($post);

            throw new InvalidArgumentException("WP_Post expected, {$class} in not an instance or has invalid properties");
        }
        $this->post = $post;
    }

    /**
     * @param WP_Post $post
     *
     * @return bool
     */
    public static function validatePost(WP_Post $post): bool
    {
        return !(property_exists(static::class, 'postType') && $post->post_type !== static::$postType);
    }

    /**
     * @param int|string|WP_Post $id
     *
     * @return null|static
     */
    public static function find($id = null): ?self
    {
        if (is_numeric($id) && $id > 0) {
            $post = get_post((int) $id);
        } elseif (is_string($id)) {
            $posts = get_posts([
                'name' => $id,
                'post_type' => static::$postType,
                'post_status' => 'publish',
                'posts_per_page' => 1,
            ]);
            $post = $posts && count($posts) === 1 ? $posts[0] : null;
        } elseif ($id instanceof WP_Post) {
            $post = $id;
        } elseif ($id instanceof static) {
            $post = $id->getPost();
        } else {
            $post = $GLOBALS['post'] ?? null;
        }

        return $post ? new static($post) : null;
    }

    /**
     * @param array $attrs
     * @param bool $byName
     * @return array|Subscription[]
     */
    public static function findAll(array $attrs, bool $byName = false): array
    {
        if ($byName) {
            $metaQuery = [];
            foreach ($attrs as $name => $value) {
                $metaQuery[] = [
                    'key' => $byName ? static::metaKey($name) : $name,
                    'value' => $value,
                    'compare' => is_array($value) ? 'IN' : '=',
                ];
            }
        } else {
            $metaQuery = $attrs;
        }
        $metaQuery['relation'] = 'and';
        $posts = get_posts([
            'post_type' => static::postType(),
            'posts_per_page' => 1,
            'meta_query' => $metaQuery,
        ]);

        $result = [];
        foreach ($posts as $post) {
            $result[] = static::find($post);
        }

        return $result;
    }

    /**
     * @return WP_Post
     */
    public function getPost(): WP_Post
    {
        return $this->post;
    }

    public static function postType(): string
    {
        return static::$postType ?? 'post';
    }

    public function __call($name, $arguments)
    {
        return call_user_func_array([$this->post, $name], $arguments);
    }

    public function __get($name)
    {
        return $this->post->{$name};
    }

    public function __set($name, $value)
    {
        $this->post->{$name} = $value;
    }

    public function __isset($name)
    {
        return property_exists($this->post, $name);
    }
}
