<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Entity;

use Rebilly\Entities\Product;
use Rebilly\Entities\SubscriptionCancellation;
use Rebilly\Rest\Collection;
use Rebilly\Wp\Member\Plugin;
use WP_Post;
use WP_User;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class SubscriptionForm
 * @package Rebilly\Wp\Member
 *
 * @mixin WP_Post
 */
class Subscription
{
    use PostProxy, HasMeta;

    protected static $postType = 'rebilly_subscription';

    protected static $meta = [
        'cancellationId' => '_rebilly_m_cancellation_id',
        'subscriptionId' => '_rebilly_m_subscription_id',
        'formId' => '_rebilly_m_form_id',
        'planIds' => '_rebilly_m_plan_ids',
        'productIds' => '_rebilly_m_product_ids',
    ];

    public static function register(): void
    {
        add_action('init', [static::class, 'init']);
    }

    public static function init(): void
    {
        register_post_type(
            static::$postType,
            [
                'label' => __('Subscription', REBILLY_WP_LANG_DOMAIN),
                'labels' => [
                    'name' => __('Subscriptions', REBILLY_WP_LANG_DOMAIN),
                    'singular_name' => __('Subscription', REBILLY_WP_LANG_DOMAIN),
                    'menu_name' => __('Subscriptions', REBILLY_WP_LANG_DOMAIN),
                    'all_items' => __('All Subscriptions', REBILLY_WP_LANG_DOMAIN),
                    'add_new' => __('Add Subscription', REBILLY_WP_LANG_DOMAIN),
                    'add_new_item' => __('Add Subscription', REBILLY_WP_LANG_DOMAIN),
                    'edit_item' => __('Edit Subscription', REBILLY_WP_LANG_DOMAIN),
                    'new_item' => __('New Subscriptions', REBILLY_WP_LANG_DOMAIN),
                    'view_item' => __('View Subscription', REBILLY_WP_LANG_DOMAIN),
                    'view_items' => __('View Subscriptions', REBILLY_WP_LANG_DOMAIN),
                    'search_items' => __('Search Subscription', REBILLY_WP_LANG_DOMAIN),
                    'not_found' => __('No Subscriptions found', REBILLY_WP_LANG_DOMAIN),
                    'not_found_in_trash' => __('No Subscriptions found in Trash', REBILLY_WP_LANG_DOMAIN),
                ],
                'description' => __('Rebilly Subscription', REBILLY_WP_LANG_DOMAIN),
                'public' => false,
                'publicly_queryable' => false,
                'show_ui' => false,
                'delete_with_user' => false,
                'show_in_rest' => false,
                'has_archive' => false,
                'show_in_menu' => false,
                'show_in_nav_menus' => false,
                'exclude_from_search' => true,
                'capability_type' => 'post',
                'map_meta_cap' => false,
                'hierarchical' => false,
                'rewrite' => ['slug' => static::$postType, 'with_front' => true],
                'query_var' => false,
                'supports' => ['title'],
            ]
        );
    }

    /**
     * @param WP_User|User|int|null $user
     * @param \Rebilly\Entities\Subscription $subscription
     * @param array $plans
     * @param array $products
     * @param Form $form
     * @param array $data
     *
     * @return Subscription|\WP_Error|null
     */
    public static function create(
        $user,
        \Rebilly\Entities\Subscription $subscription,
        array $plans,
        array $products,
        Form $form,
        array $data = []
    ) {
        $user = $user ? User::find($user) : null;
        $data = apply_filters('rebilly_subscription_data', $data);
        $id = wp_insert_post([
            'post_status' => 'publish',
            'post_title' => $subscription->getId(),
            'post_name' => $subscription->getId(),
            'post_owner' => $user ? $user->getKey() : null,
            'post_content' => json_encode([
                'data' => $data,
                'fields' => $form->getFields(),
            ]),
            'post_type' => static::postType(),
            'meta_input' => [
                static::metaKey('subscriptionId') => $subscription->getId(),
                static::metaKey('formId') => $form->getKey(),
                static::metaKey('planIds') => array_keys($plans),
                static::metaKey('productIds') => $products,
            ],
        ]);

        $subscription = static::find($id);

        do_action('rebilly_subscription_created', $subscription);

        return $subscription;
    }

    /**
     * @param array $attrs
     * @param bool $byName
     * @return Subscription|null
     */
    public static function findFor(array $attrs, bool $byName = false): ?self
    {
        if ($byName) {
            $metaQuery = [];
            foreach ($attrs as $name => $value) {
                $metaQuery[static::metaKey($name)] = $value;
            }
        } else {
            $metaQuery = $attrs;
        }
        $metaQuery['relation'] = 'and';
        $posts = get_posts([
            'post_type' => static::postType(),
            'posts_per_page' => 1,
            'meta_query' => $metaQuery,
        ]);
        if (count($posts)) {
            return static::find($posts[0]);
        }

        return null;
    }

    /**
     * @return string[]|array
     */
    public static function getCustomFields(): array
    {
        return Plugin::api()
            ->customField()
            ->allByName('subscriptions');
    }

    public function getCancellationId(): string
    {
        return (string) $this->metaGet('cancellationId', true, '');
    }

    public function getCancellation(): ?SubscriptionCancellation
    {
        return Plugin::api()
            ->subscription()
            ->getCancellation($this->getCancellationId());
    }

    /**
     * @param string|null $cancellationId
     */
    public function setCancellationId(?string $cancellationId): void
    {
        $this->metaUpdate(compact('cancellationId'), true);
    }

    /**
     * @return string
     */
    public function getSubscriptionId(): string
    {
        return (string) $this->metaGet('subscriptionId', true, '');
    }

    /**
     * @return Form|null
     */
    public function getForm(): ?Form
    {
        return Form::find($this->getFormId());
    }

    /**
     * @return string
     */
    public function getFormId(): string
    {
        return (string) $this->metaGet('formId', true, '');
    }

    /**
     * @return array|string[]
     */
    public function getPlanIds(): array
    {
        return (array) $this->metaGet('planIds', true, []);
    }

    /**
     * @return array|string[]
     */
    public function getProductNames(): array
    {
        $names = [];
        foreach ($this->getProducts() as $product) {
            $names[] = $product->getName();
        }

        return $names;
    }

    /**
     * @return Collection|Product[]
     */
    public function getProducts(): Collection
    {
        return Plugin::api()
            ->product()
            ->all($this->getProductIds(), true);
    }

    /**
     * @return array|string[]
     */
    public function getProductIds(): array
    {
        return $this->metaGet('productIds', true);
    }
}
