<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Entity;

use Rebilly\Entities\Subscription;
use Rebilly\Rest\Collection;
use Rebilly\Wp\Member\Plugin;
use Rebilly\Wp\Member\Settings;
use WP_Error;
use WP_User;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class User extends WP_User
{
    use HasMeta;

    public const CACHE_EXPIRES = 5 * 60;

    protected static $meta = [
        'customerId' => '_rebilly_m_customer_id',
        'capabilities' => '_rebilly_m_capabilities',
    ];

    public static function findByCustomerId(string $customerId): ?self
    {
        $users = get_users([
            'meta_key' => static::metaKey('customerId'),
            'meta_value' => $customerId,
            'number' => 1,
        ]);
        if (count($users ?? []) === 0) {
            return null;
        }

        return new static($users[0]);
    }

    /**
     * @param array $data
     *
     * @return User|WP_Error
     */
    public static function create(array $data)
    {
        $username = $data['username'] ?? null;
        if (!$username) {
            if (!blank($data['first_name']) || !blank($data['last_name'])) {
                $username = implode('_', [$data['first_name'], $data['last_name']]);
            } else {
                $username = $data['email'];
            }
        }
        if (strpos($username, '@') !== false) {
            [$username,] = explode('@', $username);
        }
        $username = preg_replace('/[^\w_-]/', '_', $username);
        $user = wp_insert_user([
            'user_pass' => '',
            'user_login' => $username,
            'user_email' => $data['email'],
            'display_name' => trim("{$data['first_name']} {$data['last_name']}"),
            'first_name' => $data['first_name'],
            'last_name' => $data['last_name'],
        ]);

        return !is_wp_error($user) ? static::find($user) : $user;
    }

    public static function findByEmail(string $email): ?self
    {
        $user = get_user_by('email', $email);

        return $user ? static::find($user) : null;
    }

    public static function find($id = null): ?self
    {
        if ($id === null && $current = wp_get_current_user()) {
            return $current->exists() ? new static($current) : null;
        }
        if (is_int($id)) {
            return new static(get_userdata($id));
        }
        if ($id instanceof WP_User) {
            return new static($id);
        }

        return null;
    }

    public static function getNames(): array
    {
        return wp_rebilly_m_cached('user-names', function () {
            $names = [];
            /** @var WP_User[]|array $users */
            $users = get_users();
            foreach ($users as $user) {
                $names[$user->ID] = $user->display_name;
            }

            return $names;
        }, self::CACHE_EXPIRES);
    }

    public static function getCustomFields(): array
    {
        return Plugin::api()
            ->customField()
            ->allByName('customers');
    }

    /**
     * @return array|Subscription[]|\Rebilly\Rest\Collection
     */
    public function subscriptionsList(): Collection
    {
        return Plugin::api()
            ->subscription()
            ->findActive($this->getCustomerId());
    }

    public function getCustomer()
    {
        return Plugin::api()
            ->customer()
            ->get($this->getCustomerId());
    }

    public function getCards()
    {
        return Plugin::api()
            ->paymentCard()
            ->findByCustomer($this->getCustomerId());
    }

    public function getCustomerId(): string
    {
        return (string) $this->metaGet('customerId', true, '');
    }

    public function setCustomerId($customerId): void
    {
        $this->metaUpdate(compact('customerId'), true);
    }

    public function isCapable($requirements): bool
    {
        $capabilities = $this->getCapabilities();
        $requirements = array_unique($requirements);
        $intersect_count = count(array_intersect($capabilities, $requirements));

        return $intersect_count > 0;
    }

    public function getCapabilities(): array
    {
        return $this->metaGet('capabilities', true) ?: [];
    }

    /**
     * @param RestrictableType $restrictable
     * @return bool
     */
    public function canOverride($restrictable): bool
    {
        $override_roles = Settings::get('override_roles', []);

        return count(array_intersect($this->roles, $override_roles)) > 0 ||
            in_array($this->ID, $restrictable->getMembers(), true);
    }

    public function grantCapability($capability): void
    {
        $current = $this->getCapabilities();
        if (!is_array($current)) {
            $current = is_string($current) ? [$current] : [];
        }
        $new = array_merge($current, is_array($capability) ? $capability : [$capability]);

        $this->setCapabilities($new);
    }

    public function setCapabilities($list): void
    {
        $capabilities = is_array($list) ? array_unique($list) : [];

        $this->metaUpdate(compact('capabilities'), true);
    }

    public function revokeCapability($capability): void
    {
        $current = $this->getCapabilities();
        if (!is_array($current)) {
            $current = is_string($current) ? [$current] : [];
        }
        $new = array_diff($current, is_array($capability) ? $capability : [$capability]);

        $this->setCapabilities($new);
    }

    public function hasCapability($capability): bool
    {
        $list = $this->getCapabilities();

        return in_array($capability, $list, true);
    }
}
