<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Entities\Plan;
use Rebilly\Entities\Subscriptions\PlanPricing;
use Rebilly\Wp\Member\Entity\Form as Entity;
use Rebilly\Wp\Member\Plugin;
use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasDomRenderer;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Form implements Node
{
    use HasAttributes, HasChildren, HasDomRenderer, HasShowFor;

    protected $config = [];

    /**
     * Framepay constructor.
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;

        $this->children = Reformer::buildNodes($config['children'] ?? []);

        $this->attributes['id'] = $config['id'];
        $this->attributes['class'] = $config['class'];
        $this->attributes['method'] = 'post';
        $this->attributes['action'] = esc_url(admin_url('admin-ajax.php'));
    }

    /**
     * @return string
     */
    public function getTag(): string
    {
        return 'form';
    }

    /**
     * @param array $env
     *
     * @return string
     */
    public function render(array $env): string
    {
        if (!$this->isAvailable($env)) {
            return '';
        }

        [$tag, $attributesRaw, $inner] = [$this->getTag(), $this->getAttributes(), $this->renderInner($env)];

        if (isset($env['form'])) {
            $attributesRaw['class'] = trim(($attributesRaw['class'] ?? '') . " role-{$env['form']->getRole()}");
            $attributesRaw['data-plans'] = json_encode($this->getPlanSpecifications($env['form']));
        }

        // Filter empty ones
        $attributesRaw = array_filter($attributesRaw, function ($value) {
            return !blank($value);
        });

        // Compile attributes
        $attributes = implode(' ', array_map(function ($keyRaw, $valueRaw) {
            [$key, $value] = [esc_attr($keyRaw), esc_attr($valueRaw)];

            return "$key=\"$value\"";
        }, array_keys($attributesRaw), $attributesRaw));

        return "<$tag $attributes>$inner</$tag>";
    }

    /**
     * @param array $env
     *
     * @return string
     */
    protected function renderInner(array $env): string
    {
        $formId = esc_attr($env['form']->ID);

        $inner = <<<HTML
<div class="wp_rebilly_m__overlay">
    <div class="wp_rebilly_m__logo">
        <div class="mr-re"></div>
        <div class="mr-bil"></div>
        <div class="mr-ly"></div>
    </div>
</div>
<div class="wp_rebilly_m__message_group"></div>

<input type="hidden" data-rebilly="token" name="rebilly-token" class="wp_rebilly_m__card_token">
<input type="hidden" name="_form" value="$formId">
HTML;
        if(($env['action'] ?? '') !== '') {
            $actionName = esc_attr($env['action']->getName());
            $inner .= <<<HTML
<input type="hidden" name="action" value="$actionName">
HTML;
            if (($nonce = $env['action']->generateNonce()) !== null) {
                $nonce = esc_attr($nonce);
                $inner .= <<<HTML
<input type="hidden" name="_nonce" value="$nonce">
HTML;
            }
        }

        foreach ($this->getChildren() as $child) {
            $inner .= $child->render($env);
        }

        return $inner;
    }

    protected function getPlanSpecifications(Entity $form): array
    {
        $bumpFields = array_keys($form->getBumpFields());
        $addonFieldNames = $form->getAddonFields();

        $quantityFields = $form->getPlanQuantityFields();

        $plans = $this->getPlans($form);

        $base = $this->getPlanInfo($form->getPlanIds(), $quantityFields, $plans);
        $bump = $this->getPlanInfo($form->getPlanIds(true), $quantityFields, $plans);

        $addons = [];
        foreach ($addonFieldNames as $fieldName => $field) {
            $addons[$fieldName] = array_map(function ($planId) use ($plans) {
                return [
                    'plan' => $planId,
                    'mode' => 'preset',
                    'quantity' => 1,
                    'price' => $this->getFirstPlanPricing($plans[$planId]),
                ];
            }, array_keys($form->getAddonPlanIds($field['settings']['addon'])));
        }

        return compact('base', 'bump', 'addons', 'bumpFields');
    }

    /**
     * @param array $plans
     * @param array $quantityFields
     * @param Plan[]|array $planEntities
     * @return array
     */
    protected function getPlanInfo(array $plans, array $quantityFields, array $planEntities): array
    {
        return array_map(function ($plan) use ($quantityFields, $planEntities) {
            return [
                'plan' => $plan['plan'],
                'mode' => $plan['mode'] && isset($quantityFields[$plan['plan']])
                    ? 'input'
                    : 'preset',
                'quantity' => $plan['mode']
                    ? ($quantityFields[$plan['plan']] ?? ($plan['quantity'] ?? 1))
                    : ($plan['quantity'] ?? 1),
                'price' => $this->getFirstPlanPricing($planEntities[$plan['plan']]),
            ];
        }, $plans);
    }

    /**
     * @param Entity $form
     *
     * @return Plan[]|array
     */
    protected function getPlans(Entity $form): array
    {
        $ids = array_merge(
            array_column($form->getPlanIds(), 'plan'),
            array_column($form->getPlanIds(true), 'plan'),
            array_keys($form->getAddonPlanIds())
        );

        $plansRaw = Plugin::api()
            ->plan()
            ->all($ids);

        $plans = [];
        foreach ($plansRaw as $plan) {
            $plans[$plan->getId()] = $plan;
        }

        return $plans;
    }

    protected function getFirstPlanPricing(Plan $plan): array
    {
        if ($plan->getTrial() !== null) {
            return [
                'formula' => PlanPricing::FIXED_FEE,
                'price' => $plan->getTrial()->getPrice(),
            ];
        }

        return $plan->getPricing()->jsonSerialize();
    }
}
