<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasNoChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Input implements Node
{
    use HasAttributes, HasNoChildren, HasShowFor;

    protected $config = [];

    /**
     * Node constructor.
     *
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;

        $this->attributes['id'] = $config['id'];
        $this->attributes['class'] = $config['class'];
        foreach ($config['attributes'] as $key => $value) {
            if (blank($value)) {
                continue;
            }
            if (in_array($key, ['name', 'value', 'type', 'placeholder', 'maxlength', 'min', 'step', 'max', 'data-geo-fill'], true)) {
                $this->attributes[$key] = $value;
            } elseif (in_array($key, ['required', 'checked'], true)) {
                $this->attributes[$key] = $key;
            }
        }
        $field = $config['settings']['rebilly'] ?? '';
        if (!blank($field)) {
            $this->attributes['data-rebilly'] = $field;
        }
    }

    /**
     * @return string
     */
    public function getTag(): string
    {
        return 'input';
    }

    /**
     * @param array $env
     *
     * @return string
     */
    public function render(array $env): string
    {
        if (!$this->isAvailable($env)) {
            return '';
        }

        [$tag, $attributesRaw] = [$this->getTag(), $this->getAttributes()];

        // Pre-fill fields for authenticated users
        if (isset($attributesRaw['data-rebilly'])) {
            if ($env['isAuthenticated']) {
                $value = '';
                switch ($attributesRaw['data-rebilly']) {
                    case 'firstName':
                        $value = $env['user']->user_firstname;

                        break;
                    case 'lastName':
                        $value = $env['user']->user_lastname;

                        break;
                    case 'email':
                        $value = $env['user']->user_email;

                        break;
                }
                if (!blank($value)) {
                    $attributesRaw['value'] = $value;
                    $attributesRaw['readonly'] = 'readonly';
                }
            }
            if(isset($env['preFill'][$attributesRaw['data-rebilly']])) {
                $attributesRaw['value'] = $env['preFill'][$attributesRaw['data-rebilly']];
            }
        }

        // Filter empty ones
        $attributesRaw = array_filter($attributesRaw, function ($value) {
            return !blank($value);
        });

        // Compile attributes
        $attributes = implode(' ', array_map(function ($keyRaw, $valueRaw) {
            [$key, $value] = [esc_attr($keyRaw), esc_attr($valueRaw)];

            return "$key=\"$value\"";
        }, array_keys($attributesRaw), $attributesRaw));

        return "<$tag $attributes/>";
    }
}
