<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasDomRenderer;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Select implements Node
{
    use HasAttributes, HasChildren, HasShowFor;
    use HasDomRenderer {
        render as renderDom;
        renderInner as renderInnerDom;
    }

    protected $config = [];

    protected static $presets = [
        'country' => 'wp_rebilly_m_countries',
        'state' => 'wp_rebilly_m_states',
    ];

    /**
     * Node constructor.
     *
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;

        $children = $this->buildOptions($config['settings']['options'] ?? []) ?: ($config['children'] ?? []);
        $preset = $config['settings']['data-preset'] ?? '';
        $children = array_merge($children, $this->getPreset($preset));

        $this->children = Reformer::buildNodes($children);

        $this->attributes['id'] = $config['id'];
        $this->attributes['class'] = $config['class'];
        foreach ($config['attributes'] as $key => $value) {
            if (blank($value)) {
                continue;
            }
            if (in_array($key, ['name', 'data-geo-fill'], true)) {
                $this->attributes[$key] = $value;
            } elseif ($key === 'required') {
                $this->attributes[$key] = $key;
            }
        }

        if (!blank($config['settings']['rebilly'] ?? '')) {
            $this->attributes['data-rebilly'] = $config['settings']['rebilly'];
        }
    }

    /**
     * @return string
     */
    public function getTag(): string
    {
        return 'select';
    }

    public function getOptions(): array
    {
        $options = [];
        foreach ($this->children as $option) {
            if ($option instanceof Option) {
                $options[$option->getKey()] = $option->getLabel();
            }
        }

        return $options;
    }

    public function render(array $env): string
    {
        return $this->renderDom($env);
    }

    public function renderInner(array $env): string
    {
        $env['rebilly-role'] = $this->getAttributes()['data-rebilly'] ?? null;

        return $this->renderInnerDom($env);
    }

    private function getPreset(string $preset): array
    {
        if (!array_key_exists($preset, static::$presets)) {
            return [];
        }
        $func = static::$presets[$preset];
        $presets = $func();
        $children = [];
        foreach ($presets as $key => $value) {
            $children[] = [
                'type' => 'option',
                'attributes' => ['value' => $key],
                'children' => [
                    [
                        'type' => 'textnode',
                        'content' => $value,
                    ],
                ],
            ];
        }

        return $children;
    }

    private function buildOptions(array $options): array
    {
        return array_map(function ($option) {
            return [
                'type' => 'option',
                'attributes' => ['value' => $option['value']],
                'children' => [
                    [
                        'type' => 'textnode',
                        'content' => $option['name'],
                    ],
                ],
            ];
        }, $options);
    }
}
