<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class Settings
 * @package Rebilly\Wp\Member
 *
 * @method static mixed get($key, $default = null)
 * @method static mixed set($key, $value = null)
 */
class Settings
{
    /** @var Settings */
    protected static $_instance;

    /** @var array */
    private $_settings = [];

    private function __construct()
    {
    }

    public static function instance(): self
    {
        if (!static::$_instance) {
            static::$_instance = new static();
        }

        return static::$_instance;
    }

    /**
     * @param string $key Dot notated key
     * @param mixed|null $default
     *
     * @return mixed
     */
    protected function __get__(string $key, $default = null)
    {
        $pathComponents = $this->unDot($key);
        $current = &$this->_settings;
        foreach ($pathComponents as $path) {
            if (!array_key_exists($path, $current)) {
                return $default;
            }
            $current = &$current[$path];
        }

        return $current;
    }

    /**
     * @param string $key Dot notated key
     * @param mixed|null $value
     *
     * @return mixed
     */
    protected function __set__(string $key, $value = null)
    {
        $was = $this->__get__($key);

        $pathComponents = $this->unDot($key, true);
        foreach ($pathComponents as $path) {
            $value = [$path => $value];
        }
        $this->_settings = array_replace_recursive($this->_settings, (array) $value);

        return $was;
    }

    /**
     * Split dot notation to path components
     *
     * @param string $key Dot notated key
     * @param bool $reverse Whether to reverse path components order or not
     *
     * @return array
     */
    private function unDot(string $key, bool $reverse = false): array
    {
        $result = array_filter(explode('.', $key), function ($path) {
            return !blank($path);
        });

        return $reverse ? array_reverse($result) : $result;
    }

    public function __call(string $name, array $arguments)
    {
        switch ($name) {
            case 'get':
                return $this->__get__(...$arguments);
            case 'set':
                return $this->__set__(...$arguments);
        }

        return null;
    }

    public static function __callStatic(string $name, array $arguments)
    {
        return static::instance()->{$name}(...$arguments);
    }
}
